#!/usr/bin/python
# -*- coding: utf8 -*-

import irclib
import ircbot
from threading import Thread

from gi.repository import GLib
from gi.repository import Gtk
from gi.repository import GObject
from gi.repository import Granite
from gi.repository import WebKit
from multiprocessing import Process, Lock, Queue
from multiprocessing.managers import BaseManager
import Queue as queue
from time import strftime
import os
import xdg.BaseDirectory as xdg_dirs

class TemporaryChannel(Gtk.Grid):
    def __init__(self):
        Gtk.Grid.__init__(self)

        self.channel_entry = Gtk.Entry()
        self.channel_entry.set_placeholder_text("Channel")
        self.channel_entry.set_size_request(250, -1)
        self.channel_entry.set_vexpand(True)
        self.channel_entry.set_hexpand(True)
        self.channel_entry.set_valign(Gtk.Align.CENTER)
        self.channel_entry.set_halign(Gtk.Align.END)
        self.attach(self.channel_entry, 0, 0, 1, 1)
        
        ok = Gtk.Button.new_with_label("Join")
        ok.set_vexpand(True)
        ok.set_hexpand(True)
        ok.set_valign(Gtk.Align.CENTER)
        ok.set_halign(Gtk.Align.START)
        ok.connect("clicked", self.on_activate)
        ok.connect("activate", self.on_activate)
        self.attach(ok,11, 0, 1, 1)

    def on_activate(self, widget):
        for w in self.get_children():
            self.remove(w)
        self.server.mute_temporary_to_real(self)

    def get_channel(self):
        return self.channel_entry.get_text()

class Channel(object):
    def __init__(self, server, name):
        self.server = server
        self.name = name
        self.webkit = WebKit.WebView.new()
        self.pending_items = []
        self.loaded = False
        self.webkit.connect("load-finished", self.on_load_finished)
        content = GLib.file_get_contents ("/usr/share/adium/message-styles/Postler.AdiumMessageStyle/Contents/Resources/Template.html")[1]
        content = content.replace("%@", "#", 1)
        content = content.replace("%@", "main.css", 1)
        content = content.replace("%@", "", 2)
        self.webkit.load_html_string(content, "file:///usr/share/adium/message-styles/Postler.AdiumMessageStyle/Contents/Resources/Template.html")
        self.webview = Gtk.ScrolledWindow(None, None)
        self.webview.add(self.webkit)
        self.webview.set_vexpand(True)
        self.webview.set_hexpand(True)

    def on_load_finished(self, frame, data):
        self.loaded = True
        for i in self.pending_items:
            self.add_message(i)
    
    def add_message(self, message):
        if self.loaded:
            author = message[1]
            time = message[2]
            content_m = message[3].replace("'", "\\'")
            channel = message[4]
            content = GLib.file_get_contents("/usr/share/adium/message-styles/Postler.AdiumMessageStyle/Contents/Resources/Outgoing/Content.html")[1]
            content = content.replace("\n", "")
            content = content.replace("%message%", content_m)
            content = content.replace("%sender%", author)
            content = content.replace("%time{%X}%", time)
            
            self.webkit.execute_script("appendMessage('" + content + "');")
        else:
            self.pending_items.append(message)


class MessageManager(object):
    def __init__(self):
        self.pending_items = []

    def append_message (self, author, time, content, channel, server):
        self.pending_items.append ((0, irclib.nm_to_n(author), time, content, channel, server))
        
        # Let's save it
        os.system ("mkdir " + xdg_dirs.xdg_data_home + "/cable/ -p") # not so clean
        log_file = open(xdg_dirs.xdg_data_home + "/cable/" + server + "-" + channel, "a")
        log_file.write ("<" + time + "> " + author + ": " + content + "\n")
        log_file.close()

    def get_pending_items(self):
        items = self.pending_items
        self.pending_items = []
        return items

    def check(self):
        try:
            a = self.q.get(False)
            self.append_message (a[0], a[1], a[2], a[3], a[4])
        except queue.Empty:
            pass

class IRCServer(ircbot.SingleServerIRCBot):
    def __init__(self, adress, port, username):
        self.server = adress
        ircbot.SingleServerIRCBot.__init__(self, [(adress, port)],
                                           username, ";)")

    def on_welcome(self, serv, ev):
        serv.execute_delayed (0.5, self.update_event, (serv,))
        pass

    def update_event (self, serv):
        try:
            data = self.queue_irc.get(False)
            if data[0] == 0: # channel request
                serv.join(data[1])
            elif data[0] == 1: # message
                if data[2][0] == '/':
                    serv.send_raw(data[2])
                    print data[2]
                else:
                    serv.privmsg(data[1], data[2])
        except queue.Empty:
            pass
        serv.execute_delayed (0.5, self.update_event, (serv,))

    def on_pubmsg(self, serv, ev):
        self.q.put ((ev.source(), strftime("%H:%M:%S"), ev.arguments()[0], ev.target(), self.server))
    def on_privmsg(self, serv, ev):
        self.q.put ((ev.source(), strftime("%H:%M:%S"), ev.arguments()[0], irclib.nm_to_n(ev.source()), self.server))

class Server(object):
    def __init__(self, adress, port):
        self.adress = adress
        self.port = port
        self.nickname = "xapantu__"
        self.irc = IRCServer (adress, port, self.nickname)
        self.queue = Queue()
        self.manager = MessageManager()
        self.manager.q = self.queue
        self.queue_irc = Queue()

        self.channels = {}
        
        self.t = Process(target=self.process_start, args=(self.queue,self.queue_irc,))
        self.t.start()
        
        # Let's view...
        self.dynamic_notebook = Granite.WidgetsDynamicNotebook.new()
        self.dynamic_notebook.connect("add-button-clicked", self.add_button_clicked)
        self.dynamic_notebook.server = self

    def add_button_clicked(self, notebook):
        widget = TemporaryChannel()
        widget.server = self
        tab = notebook.append_page(widget, "New Tab", "gtk-file")
        widget.tab = tab

    def mute_temporary_to_real(self, widget):
        name = widget.get_channel()
        self.queue_irc.put((0, name))
        self.channels[name] = Channel(self.adress, name)
        widget.attach(self.channels[name].webview, 0, 0, 1, 1)
        widget.tab.set_text(name)
        self.dynamic_notebook.show_all ()

    def process_start(self, queue, queue_irc):
        self.irc.q = queue
        self.irc.queue_irc = queue_irc
        self.irc.start()

    def new_channel(self, name):
        self.queue_irc.put((0, name))
        self.channels[name] = Channel(self.adress, name)
        self.dynamic_notebook.append_page (self.channels[name].webview, name, "gtk-file")

    def check_new_message(self):
        self.manager.check()
        for message in self.manager.get_pending_items():
            channel = message[4]
            if channel in self.channels:
                self.channels[channel].add_message(message)
            else:
                self.new_channel(channel)
                self.channels[channel].add_message(message)

    def send_message(self, channel, content):
        self.queue_irc.put((1, channel, content))
        self.manager.append_message(self.nickname, strftime("%H:%M:%S"), content, channel, self.adress)


class Cable(Granite.Application):
    def __init__(self):
        Granite.Application.__init__(self, application_id="org.elementary.cable2")
        self.connect("activate", self.on_activate)
        self.popover = None


    def on_loop(self):
        for s in self.servers.values():
            s.check_new_message ()
        return True

    def on_modebutton_changed (self, modebutton, widget):
        self.notebook.set_current_page(modebutton.get_selected())

    def open_server(self, adress, name, port=6667):
        self.servers[name] = Server(adress, port)
        self.notebook.append_page(self.servers[name].dynamic_notebook, None)
        self.mode_button.append (Gtk.Label(name))
        self.mode_button.set_selected(self.notebook.page_num(self.servers[name].dynamic_notebook))
        self.win.show_all()
        self.on_modebutton_changed(self.mode_button, None)

    def on_activate(self, data=None):
        self.servers = {}
        self.servers["localhost"] = Server("localhost", 6667)

        GLib.timeout_add (50, self.on_loop)
        
        self.mode_button = Granite.WidgetsModeButton.new()
        self.notebook = Gtk.Notebook()
        self.mode_button.connect("mode-changed", self.on_modebutton_changed)
        self.mode_button.append (Gtk.Label("localhost"))

        self.toolbar = Gtk.Toolbar()
        toolitem = Gtk.ToolItem()
        toolitem.add(self.mode_button)
        expander = Gtk.ToolItem()
        expander.set_expand(True)
        self.toolbar.add(expander)
        grid = Gtk.Grid()
        grid.attach(self.toolbar, 0, 0, 1, 1)
        self.toolbar.hexpand = True
        self.toolbar.vexpand = False
        self.toolbar.get_style_context().add_class("primary-toolbar")
        self.toolbar.add (toolitem)
        button_server = Gtk.ToolButton.new_from_stock("gtk-connect")
        button_server.connect("clicked", self.on_new_server)
        expander = Gtk.ToolItem()
        expander.set_expand(True)
        self.toolbar.add(expander)
        self.toolbar.add(button_server)

        self.win = Gtk.Window.new(Gtk.WindowType.TOPLEVEL)
        self.win.connect("delete-event", self.on_destroy)
        self.win.add(grid)
        self.notebook = Gtk.Notebook()
        self.notebook.set_show_tabs(False)
        self.notebook.append_page(self.servers["localhost"].dynamic_notebook, None)
        grid.attach(self.notebook, 0, 1, 1, 1)
        self.entry = Gtk.Entry()
        grid.attach(self.entry, 0, 2, 1, 1)
        self.entry.connect("activate", self.on_entry_activate)
        self.add_window(self.win)
        self.win.show_all()
        self.win.resize(800, 600)

    def on_new_server(self, button):
        if self.popover is not None: self.popover.destroy()
        popover = Granite.WidgetsPopOver.new()
        self.popover_entry_adress = Granite.WidgetsHintedEntry.new("Adress")
        self.popover_entry_name = Granite.WidgetsHintedEntry.new("Name")
        popover.get_content_area ().add(self.popover_entry_adress)
        popover.get_content_area ().add(self.popover_entry_name)
        popover.add_button ("gtk-ok", Gtk.ResponseType.OK)
        popover.set_modal(False)
        popover.show_all()
        popover.move_to_widget(button)
        popover.present()
        popover.connect("response", self.on_popover_response)
        self.popover = popover

    def on_popover_response(self, popover, response):
        if response == Gtk.ResponseType.OK:
            self.open_server (self.popover_entry_adress.get_text(), self.popover_entry_name.get_text())
            self.popover.destroy ()

    def on_entry_activate(self, entry):
        #good notebook:
        server = self.notebook.get_nth_page(self.notebook.get_current_page()).server
        #good tab:
        channel = server.channels.values() [server.dynamic_notebook.get_page()]
        print channel.name
        server.send_message (channel.name, entry.get_text())

    def on_destroy(self, data=None, data2=None):
        self.win.destroy()

    def terminate_all_process(self):
        for s in self.servers.values():
            s.t.terminate()

GObject.type_register(Cable)            

if __name__ == "__main__":
    GObject.threads_init()
    cable = Cable()
    cable.run("", 0)
    cable.terminate_all_process ()


